import pandas as pd

from ovretl.shipments_utils.shipment_accepted_proposition_total_transit_time import (
    shipments_accepted_proposition_total_transit_time,
)


def test_shipments_accepted_proposition_total_transit_time():
    sh_df = pd.DataFrame(
        data={
            "shipment_id": ["1", "2", "3", "4"],
            "shipment_status": ["booked", "awaiting_booking", "finished", "cancelled"],
            "proposition_status": ["accepted", "sent", "accepted", "accepted"],
            "transit_time_door_to_port": 1,
            "transit_time": 2,
            "transit_time_port_to_door": 3,
        }
    )
    result_should_be = sh_df[sh_df["shipment_id"] == "1"]
    result_should_be["total_transit_time"] = 6
    result = shipments_accepted_proposition_total_transit_time(sh_df)
    pd.testing.assert_frame_equal(result, result_should_be)



import pandas as pd


def check_shipment_in_progress(shipment: pd.Series, events_shipment_df: pd.DataFrame) -> str:
    if shipment["shipment_status"] != "booked":
        return shipment["shipment_status"]
    shipment_events = events_shipment_df[events_shipment_df["shipment_id"] == shipment["shipment_id"]]
    if len(shipment_events) == 0:
        return "booked"
    all_events_are_done = (shipment_events["date_type"] == "actual").all()
    at_least_one_event_is_done = (shipment_events["date_type"] == "actual").any()
    if not all_events_are_done and at_least_one_event_is_done:
        return "in_progress"
    return "booked"



from datetime import datetime

import numpy as np
import pandas as pd
from ovretl.shipments_utils.match_shipments_propositions import match_shipments_propositions
from pandas.util.testing import assert_frame_equal


def test_match_shipments_propositions():
    shipments_df = pd.DataFrame(data={"id": [1, 2], "status": ["booked", "finished"], "transit_time": [2, 3]})
    propositions_df = pd.DataFrame(
        data={
            "id": [1, 2, 3, 4],
            "created_at": [datetime(2020, 5, 2), datetime(2020, 5, 1), datetime(2020, 5, 1), datetime(2020, 5, 4),],
            "shipment_id": [1, 1, 2, 2],
            "status": ["purchase_ready", "sent", "new", "new"],
            "purchase_ready_date": [0, 0, 0, 0],
            "sent_date": [0, 0, 0, 0],
            "accepted_date": [0, 0, 0, 0],
            "transit_time": [0, 0, 0, 0],
            "transit_time_door_to_port": [0, 0, 0, 0],
            "transit_time_port_to_door": [0, 0, 0, 0],
            "kronos_state": [np.nan, "sent", "auto", "auto"],
        }
    )
    result = match_shipments_propositions(shipments_df=shipments_df, propositions_df=propositions_df)
    result_should_be = pd.DataFrame(
        data={
            "shipment_id": [1, 2],
            "shipment_status": ["booked", "finished"],
            "proposition_id": [2, 4],
            "proposition_status": ["sent", "new"],
            "purchase_ready_date": [0, 0],
            "sent_date": [0, 0],
            "accepted_date": [0, 0],
            "transit_time": [0, 0],
            "transit_time_door_to_port": [0, 0],
            "transit_time_port_to_door": [0, 0],
            "kronos_state": ["sent", "auto"],
        }
    )
    assert_frame_equal(result, result_should_be)



import pandas as pd


def match_computed_quotations_and_shipments(
    computed_quotations_df: pd.DataFrame, shipments_all_propositions_df: pd.DataFrame
):
    computed_quotations_df = computed_quotations_df.rename(columns={"id": "computed_quotation_id"})
    computed_quotations_df = computed_quotations_df.dropna(subset=["shipment_id"])

    # Add a foreign key, "foresea_name", to mark computed quotations which are the most relevant
    computed_quotations_df = pd.merge(
        computed_quotations_df,
        shipments_all_propositions_df[["proposition_id", "kronos_state"]].dropna(subset=["proposition_id"]),
        how="left",
        on="proposition_id",
    )

    # Sort with foresea_name first to put most relevant CQs first
    computed_quotations_df = computed_quotations_df.sort_values(
        by=["shipment_id", "proposition_id", "created_at"], ascending=False,
    )
    computed_quotations_df = computed_quotations_df.drop_duplicates(subset=["shipment_id"])
    return computed_quotations_df.drop(["foresea_name"], axis=1, errors="ignore")



import pandas as pd
import datetime
import numpy as np


def total_tasks_weight(tasks_df: pd.DataFrame) -> pd.DataFrame:
    tasks_df = tasks_df.loc[:, ["shipment_id", "weight"]].groupby("shipment_id").sum()
    tasks_df.loc[:, "weight"] = tasks_df["weight"].fillna(0)
    return tasks_df.rename(columns={"weight": "total_tasks_weight"})


def daily_tasks_total_weight_on_shipment(tasks_df: pd.DataFrame, date: datetime.date) -> int:
    is_todo_task_due_date = (tasks_df["status"] == "to_do") & (tasks_df["due_date"] == date)
    is_done_task_done_date = (tasks_df["status"] == "done") & (tasks_df["updated_at"] == date)
    tasks_todo_today_on_shipment_df = tasks_df[(is_todo_task_due_date | is_done_task_done_date)]
    if len(tasks_todo_today_on_shipment_df["shipment_id"]) == 0:
        return 0
    daily_tasks_todo_total_weight_df = (
        tasks_todo_today_on_shipment_df.loc[:, ["shipment_id", "weight"]].groupby("shipment_id").sum()
    )
    return daily_tasks_todo_total_weight_df.weight.iat[0]


def shipment_total_workflows_weight(active_shipments_data_df: pd.DataFrame,) -> pd.DataFrame:
    shipment_total_workflows_weight_df: pd.DataFrame = active_shipments_data_df.loc[
        :, ["shipment_id", "total_workflow_weight"]
    ].groupby("shipment_id").sum()
    shipment_total_workflows_weight_df = shipment_total_workflows_weight_df.rename(
        columns={"total_workflow_weight": "total_workflows_weight"}
    )
    return shipment_total_workflows_weight_df


def find_shipment_workflow_start_date(active_shipments_data_df: pd.DataFrame) -> pd.DataFrame:
    active_shipments_data_df = active_shipments_data_df.groupby("shipment_id").min()
    active_shipments_data_df.loc[:, "workflow_associated_at"] = active_shipments_data_df[
        "workflow_associated_at"
    ].dt.date
    return active_shipments_data_df.rename(columns={"workflow_associated_at": "workflow_start_date"})


def shipments_remaining_days(active_shipments_data_df: pd.DataFrame, today: datetime.date) -> pd.DataFrame:
    active_shipments_data_df = active_shipments_data_df[
        [
            "shipment_id",
            "shipment_status",
            "freight_method",
            "freight_type",
            "foresea_name",
            "incoterm",
            "import_or_export",
            "total_transit_time",
            "workflow_associated_at",
        ]
    ]
    shipments_with_start_dates_df = find_shipment_workflow_start_date(active_shipments_data_df)
    shipments_with_start_dates_df.loc[:, "remaining_days"] = shipments_with_start_dates_df["total_transit_time"] - (
        today - shipments_with_start_dates_df["workflow_start_date"]
    ).apply(lambda delta: delta.days)
    shipments_with_start_dates_df.loc[:, "remaining_days"] = shipments_with_start_dates_df["remaining_days"].clip(
        lower=0
    )
    return shipments_with_start_dates_df


def shipments_timelines(active_shipments_data_df: pd.DataFrame) -> pd.DataFrame:
    shipments_timelines_list = []
    for index, row in active_shipments_data_df.iterrows():
        shipment_timeline_df = pd.DataFrame(data={"shipment_id": [], "dates": []})
        start_date = row["workflow_start_date"]
        transit_time = row["total_transit_time"]
        dates = pd.date_range(start=start_date, periods=transit_time)
        shipment_timeline_df["dates"] = dates
        shipment_timeline_df["shipment_id"] = index
        shipments_timelines_list.append(shipment_timeline_df)
    shipments_timelines_df = pd.concat(shipments_timelines_list)
    shipments_timelines_df.loc[:, "dates"] = shipments_timelines_df["dates"].dt.date
    return shipments_timelines_df


def estimated_remaining_daily_workload(
    active_shipments_data_df: pd.DataFrame, tasks_df: pd.DataFrame, today: datetime.date,
) -> pd.DataFrame:
    shipment_total_workflows_weight_df = shipment_total_workflows_weight(
        active_shipments_data_df=active_shipments_data_df,
    )
    active_shipments_data_df = shipments_remaining_days(active_shipments_data_df, today)
    total_tasks_weight_df = total_tasks_weight(tasks_df=tasks_df)
    active_shipments_data_df = pd.merge(active_shipments_data_df, total_tasks_weight_df, on="shipment_id", how="left")
    active_shipments_data_df = pd.merge(
        active_shipments_data_df, shipment_total_workflows_weight_df, on="shipment_id", how="left"
    )
    active_shipments_data_df.loc[:, "estimated_remaining_daily_workload"] = (
        active_shipments_data_df["total_workflows_weight"] - active_shipments_data_df["total_tasks_weight"]
    ) / active_shipments_data_df["remaining_days"]
    active_shipments_data_df.loc[
        ~np.isfinite(active_shipments_data_df["estimated_remaining_daily_workload"]),
        "estimated_remaining_daily_workload",
    ] = 0
    return active_shipments_data_df


def daily_workload_calculation(shipments_timelines_df: pd.DataFrame, tasks_df: pd.DataFrame) -> pd.DataFrame:
    shipments_timelines_df.loc[:, "created_tasks_weight"] = shipments_timelines_df.apply(
        lambda row: daily_tasks_total_weight_on_shipment(
            tasks_df=tasks_df.loc[tasks_df["shipment_id"] == row["shipment_id"], :], date=row["dates"],
        ),
        axis=1,
    )
    shipments_timelines_df.loc[:, "daily_workload"] = (
        shipments_timelines_df["created_tasks_weight"] + shipments_timelines_df["estimated_remaining_daily_workload"]
    )
    return shipments_timelines_df


def shipments_daily_workload_timelines(
    active_shipments_data_df: pd.DataFrame, tasks_df: pd.DataFrame, today: datetime.date,
) -> pd.DataFrame:
    active_shipments_data_df.loc[:, ["workflow_associated_at"]] = active_shipments_data_df[
        "workflow_associated_at"
    ].apply(lambda datestr: datetime.datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S.%fZ"))

    tasks_df = tasks_df.loc[:, ["shipment_id", "due_date", "status", "weight", "updated_at"]]
    active_shipments_data_df = estimated_remaining_daily_workload(
        active_shipments_data_df=active_shipments_data_df, tasks_df=tasks_df, today=today,
    )
    shipments_timelines_df = shipments_timelines(active_shipments_data_df)
    shipments_timelines_df = pd.merge(shipments_timelines_df, active_shipments_data_df, how="left", on="shipment_id")
    shipments_timelines_df.loc[shipments_timelines_df["dates"] < today, "estimated_remaining_daily_workload"] = 0.0
    shipments_timelines_df = daily_workload_calculation(
        shipments_timelines_df=shipments_timelines_df, tasks_df=tasks_df
    )
    return shipments_timelines_df[["shipment_id", "dates", "daily_workload"]]



import pandas as pd


def shipments_accepted_proposition_total_transit_time(shipments_all_propositions_df: pd.DataFrame) -> pd.DataFrame:
    active_shipments = shipments_all_propositions_df[
        ~shipments_all_propositions_df["shipment_status"].isin(["finished", "cancelled"])
    ]
    shipments_accepted_proposition_df = active_shipments[
        shipments_all_propositions_df["proposition_status"] == "accepted"
    ]
    shipments_accepted_proposition_df.loc[
        :, ["transit_time_door_to_port", "transit_time_port_to_door", "transit_time"]
    ] = shipments_accepted_proposition_df[
        ["transit_time_door_to_port", "transit_time_port_to_door", "transit_time"]
    ].fillna(
        0
    )
    shipments_accepted_proposition_df.loc[:, "total_transit_time"] = (
        shipments_accepted_proposition_df["transit_time_door_to_port"]
        + shipments_accepted_proposition_df["transit_time"]
        + shipments_accepted_proposition_df["transit_time_port_to_door"]
    )
    return shipments_accepted_proposition_df



import numpy as np
import pandas as pd

from ovretl import match_computed_quotations_and_shipments


def test_match_computed_quotations_and_shipments():
    cq_df = pd.DataFrame(
        data={
            "id": [1, 2, 3, 4, 5, 6],
            "shipment_id": [0, 0, 1, 1, 2, 2],
            "proposition_id": [1, np.nan, 5, np.nan, np.nan, np.nan],
            "created_at": [1, 2, 3, 4, 6, 5],
        }
    )
    shipments_df = pd.DataFrame(
        data={
            "id": [1, 2, 3, 4, 10],
            "proposition_id": [1, 2, 3, 4, np.nan],
            "kronos_state": ["sent", np.nan, "auto", np.nan, "auto"],
        }
    )
    result_should_be = pd.DataFrame(
        data={
            "computed_quotation_id": [5, 3, 1],
            "shipment_id": [2, 1, 0],
            "proposition_id": [np.nan, 5, 1],
            "created_at": [6, 3, 1],
            "kronos_state": [np.nan, np.nan, "sent"],
        }
    )
    result = match_computed_quotations_and_shipments(cq_df, shipments_df)
    pd.testing.assert_frame_equal(result.reset_index(drop=True), result_should_be.reset_index(drop=True))



import pandas as pd


def determine_import_status(shipment: pd.Series):
    if shipment["freight_method"] == "truck":
        return "inland"
    pickup_country_is_france = not pd.isna(shipment["pickup_country"]) and shipment["pickup_country"].lower() == "fr"
    delivery_country_is_france = (
        not pd.isna(shipment["delivery_country"]) and shipment["delivery_country"].lower() == "fr"
    )
    pickup_country_in_europe = not pd.isna(shipment["pickup_country"]) and shipment["pickup_country"].lower() in [
        "fr",
        "be",
        "es",
    ]
    pickup_harbor_in_europe = not pd.isna(shipment["pickup_harbor_country"]) and shipment[
        "pickup_harbor_country"
    ].lower() in ["fr", "be", "es", "nl", "it"]
    delivery_country_in_europe = not pd.isna(shipment["delivery_country"]) and shipment["delivery_country"].lower() in [
        "fr",
        "be",
        "es",
    ]
    delivery_harbor_in_europe = not pd.isna(shipment["delivery_harbor_country"]) and shipment[
        "delivery_harbor_country"
    ].lower() in ["fr", "be", "es", "nl", "it"]

    if (delivery_country_in_europe or delivery_harbor_in_europe) and not pickup_country_is_france:
        return "import"
    if (pickup_country_in_europe or pickup_harbor_in_europe) and not delivery_country_is_france:
        return "export"
    return "cross_trade"



import pandas as pd


def so_tasks_filter(tasks_df: pd.DataFrame) -> pd.DataFrame:
    so_tasks_df = tasks_df.loc[tasks_df["so_id"].notnull(), :]
    so_tasks_df.loc[:, "created_at"] = so_tasks_df["created_at"].dt.date
    so_tasks_df.loc[:, "updated_at"] = so_tasks_df["updated_at"].dt.date
    so_tasks_df.loc[:, "due_date"] = so_tasks_df["due_date"].dt.date
    return so_tasks_df



import datetime


def get_days_from_timedelta(delta: datetime.timedelta):
    return delta.days


def set_negative_to_zero(a: float):
    if a < 0:
        return 0
    else:
        return a


def datetime_to_date(dt: datetime.datetime) -> datetime.date:
    return dt.date()



import pandas as pd


def merge_with_propositions_containers_then_with_shipments_containers(
    shipments_df: pd.DataFrame, propositions_containers_df: pd.DataFrame, shipments_containers_df: pd.DataFrame,
) -> pd.DataFrame:
    shipments_df = pd.merge(
        shipments_df,
        propositions_containers_df[["teus", "tc", "hazardous", "refrigerated", "proposition_id"]],
        how="left",
        on="proposition_id",
    )

    shipments_df_with_teus = shipments_df[~shipments_df["teus"].isnull()]
    shipments_df_without_teus = shipments_df[shipments_df["teus"].isnull()].drop(
        ["teus", "tc", "hazardous", "refrigerated"], axis=1
    )

    shipments_df_without_teus = pd.merge(
        shipments_df_without_teus,
        shipments_containers_df[["teus", "tc", "hazardous", "refrigerated", "shipment_id"]],
        how="left",
        on="shipment_id",
    )
    return (
        pd.concat([shipments_df_with_teus, shipments_df_without_teus], sort=False)
        .reset_index(drop=True)
        .rename(columns={"hazardous": "containers_hazardous", "refrigerated": "containers_refrigerated",})
    )


def split_propositions_shipments_containers(containers_df: pd.DataFrame):
    propositions_containers_df = containers_df[~containers_df["proposition_id"].isnull()]
    shipments_containers_df = containers_df[~containers_df["shipment_id"].isnull()]
    return propositions_containers_df, shipments_containers_df


def merge_shipments_with_containers(
    shipments_df: pd.DataFrame, containers_df: pd.DataFrame, drop_duplicate_key="shipment_id",
):
    (propositions_containers_df, shipments_containers_df,) = split_propositions_shipments_containers(containers_df)
    shipments_df = merge_with_propositions_containers_then_with_shipments_containers(
        shipments_df=shipments_df,
        propositions_containers_df=propositions_containers_df,
        shipments_containers_df=shipments_containers_df,
    )
    shipments_df = shipments_df.sort_values(by=["teus", "tc"], ascending=False)
    shipments_df = shipments_df[
        (~shipments_df.duplicated(subset=[drop_duplicate_key])) | (shipments_df[drop_duplicate_key].isnull())
    ]
    return shipments_df



import os

import pandas as pd
from ovretl.prices_utils.add_prices_to_shipments import add_prices_to_shipments
from pandas.util.testing import assert_frame_equal

columns_to_test = [
    "foresea_name",
    "margin_without_insurance",
    "margin_insurance",
    "turnover",
    "departure_truck_freight_price",
    "departure_fees_price",
    "freight_price",
    "arrival_fees_price",
    "arrival_truck_freight_price",
    "customs_price",
    "insurance_price",
    "vat_price",
    "other_price",
    "departure_truck_freight_purchase_price",
    "departure_fees_purchase_price",
    "freight_purchase_price",
    "arrival_fees_purchase_price",
    "arrival_truck_freight_purchase_price",
    "insurance_purchase_price",
]

cwd = os.path.dirname(__file__)


def test_add_prices_to_shipments():
    shipments_df = pd.read_csv(os.path.join(cwd, "shipments_loads.csv"))
    final_checks_df = pd.read_csv(os.path.join(cwd, "final_checks_clean.csv"))
    billings_df = pd.read_csv(os.path.join(cwd, "billings_clean.csv"))
    prices_final_check_by_category_df = pd.read_csv(os.path.join(cwd, "prices_final_check_by_category.csv"))
    prices_billings_by_category_df = pd.read_csv(os.path.join(cwd, "prices_billings_by_category.csv"))
    prices_propositions_by_category_df = pd.read_csv(os.path.join(cwd, "prices_by_category.csv"))
    result = add_prices_to_shipments(
        shipments_df=shipments_df,
        final_checks_df=final_checks_df,
        billings_df=billings_df,
        prices_final_check_by_category_df=prices_final_check_by_category_df,
        prices_billings_by_category_df=prices_billings_by_category_df,
        prices_propositions_by_category_df=prices_propositions_by_category_df,
    )
    result_should_be = pd.read_csv(os.path.join(cwd, "result_should_be.csv"))

    assert_frame_equal(
        result[columns_to_test].sort_values("foresea_name").reset_index(drop=True),
        result_should_be[columns_to_test].sort_values("foresea_name").reset_index(drop=True).fillna(0),
    )



import numpy as np
import psycopg2


def nan_to_null(
    f, _NULL=psycopg2.extensions.AsIs("NULL"), _NaN=np.NaN, _Float=psycopg2.extensions.Float,
):
    psycopg2.extensions.register_adapter(float, nan_to_null)
    if not np.isnan(f):
        return _Float(f)
    return _NULL
